@@ -1,44 +1,45 @@
-use super::chdman::*;
+use super::chdman;
 use super::checksum::*;
 use super::database::*;
-use super::maxcso::*;
+use super::dolphin;
+use super::maxcso;
 use super::model::*;
 use super::prompt::*;
-use super::sevenzip::*;
+use super::sevenzip;
 use super::util::*;
 use async_std::path::Path;
-use clap::{App, Arg, ArgMatches, SubCommand};
+use clap::{App, Arg, ArgMatches};
 use indicatif::ProgressBar;
 use simple_error::SimpleResult;
 use sqlx::sqlite::SqliteConnection;
 use std::collections::HashMap;
 use std::convert::TryFrom;
 
-pub fn subcommand<'a, 'b>() -> App<'a, 'b> {
-    SubCommand::with_name("check-roms")
-        .about("Checks ROM files integrity")
+pub fn subcommand<'a>() -> App<'a> {
+    App::new("check-roms")
+        .about("Check ROM files integrity")
         .arg(
-            Arg::with_name("ALL")
-                .short("a")
+            Arg::new("ALL")
+                .short('a')
                 .long("all")
-                .help("Checks all systems")
+                .help("Check all systems")
                 .required(false),
         )
         .arg(
-            Arg::with_name("SIZE")
-                .short("s")
+            Arg::new("SIZE")
+                .short('s')
                 .long("size")
-                .help("Recalculates ROM file sizes")
+                .help("Recalculate ROM file sizes")
                 .required(false),
         )
 }
 
 pub async fn main(
     connection: &mut SqliteConnection,
-    matches: &ArgMatches<'_>,
+    matches: &ArgMatches,
     progress_bar: &ProgressBar,
 ) -> SimpleResult<()> {
-    let systems = prompt_for_systems(connection, None, matches.is_present("ALL")).await?;
+    let systems = prompt_for_systems(connection, None, false, matches.is_present("ALL")).await?;
     for system in systems {
         progress_bar.println(&format!("Processing \"{}\"", system.name));
         check_system(
@@ -97,8 +98,24 @@ async fn check_system(
                 roms.get(0).unwrap(),
             )
             .await;
+        } else if RVZ_EXTENSION == romfile_extension {
+            result = check_rvz(
+                &mut transaction,
+                progress_bar,
+                &header,
+                &romfile_path,
+                roms.get(0).unwrap(),
+            )
+            .await;
         } else {
-            result = check_other(progress_bar, &header, &romfile_path, roms.get(0).unwrap()).await;
+            result = check_other(
+                &mut transaction,
+                progress_bar,
+                &header,
+                &romfile_path,
+                roms.get(0).unwrap(),
+            )
+            .await;
         }
 
         if result.is_err() {
@@ -126,7 +143,7 @@ async fn check_archive<P: AsRef<Path>>(
     romfile_path: &P,
     mut roms: Vec<Rom>,
 ) -> SimpleResult<()> {
-    let sevenzip_infos = parse_archive(progress_bar, romfile_path)?;
+    let sevenzip_infos = sevenzip::parse_archive(progress_bar, romfile_path)?;
 
     if sevenzip_infos.len() != roms.len() {
         bail!("Archive contains a different number of ROM files");
@@ -137,7 +154,7 @@ async fn check_archive<P: AsRef<Path>>(
         let crc: String;
         if header.is_some() || sevenzip_info.crc.is_empty() {
             let tmp_directory = create_tmp_directory(connection).await?;
-            let extracted_path = extract_files_from_archive(
+            let extracted_path = sevenzip::extract_files_from_archive(
                 progress_bar,
                 romfile_path,
                 &[&sevenzip_info.path],
@@ -145,7 +162,8 @@ async fn check_archive<P: AsRef<Path>>(
             )?
             .remove(0);
             let size_crc =
-                get_file_size_and_crc(progress_bar, &extracted_path, &header, 1, 1).await?;
+                get_file_size_and_crc(connection, progress_bar, &extracted_path, header, 1, 1)
+                    .await?;
             size = size_crc.0;
             crc = size_crc.1;
         } else {
@@ -178,7 +196,7 @@ async fn check_chd<P: AsRef<Path>>(
         .iter()
         .map(|rom| (rom.name.as_str(), rom.size as u64))
         .collect();
-    let bin_paths = extract_chd_to_multiple_tracks(
+    let bin_paths = chdman::extract_chd_to_multiple_tracks(
         progress_bar,
         romfile_path,
         &tmp_directory.path(),
@@ -188,8 +206,15 @@ async fn check_chd<P: AsRef<Path>>(
     .await?;
     let mut crcs: Vec<String> = Vec::new();
     for (i, bin_path) in bin_paths.iter().enumerate() {
-        let (_, crc) =
-            get_file_size_and_crc(progress_bar, &bin_path, &header, i, bin_paths.len()).await?;
+        let (_, crc) = get_file_size_and_crc(
+            connection,
+            progress_bar,
+            &bin_path,
+            header,
+            i,
+            bin_paths.len(),
+        )
+        .await?;
         crcs.push(crc);
     }
 
@@ -208,8 +233,26 @@ async fn check_cso<P: AsRef<Path>>(
     rom: &Rom,
 ) -> SimpleResult<()> {
     let tmp_directory = create_tmp_directory(connection).await?;
-    let iso_path = extract_cso(progress_bar, romfile_path, &tmp_directory.path())?;
-    let (size, crc) = get_file_size_and_crc(progress_bar, &iso_path, &header, 1, 1).await?;
+    let iso_path = maxcso::extract_cso(progress_bar, romfile_path, &tmp_directory.path())?;
+    let (size, crc) =
+        get_file_size_and_crc(connection, progress_bar, &iso_path, header, 1, 1).await?;
+    if i64::try_from(size).unwrap() != rom.size || crc != rom.crc {
+        bail!("CRC or size mismatch");
+    };
+    Ok(())
+}
+
+async fn check_rvz<P: AsRef<Path>>(
+    connection: &mut SqliteConnection,
+    progress_bar: &ProgressBar,
+    header: &Option<Header>,
+    romfile_path: &P,
+    rom: &Rom,
+) -> SimpleResult<()> {
+    let tmp_directory = create_tmp_directory(connection).await?;
+    let iso_path = dolphin::extract_rvz(progress_bar, romfile_path, &tmp_directory.path())?;
+    let (size, crc) =
+        get_file_size_and_crc(connection, progress_bar, &iso_path, header, 1, 1).await?;
     if i64::try_from(size).unwrap() != rom.size || crc != rom.crc {
         bail!("CRC or size mismatch");
     };
@@ -217,12 +260,14 @@ async fn check_cso<P: AsRef<Path>>(
 }
 
 async fn check_other<P: AsRef<Path>>(
+    connection: &mut SqliteConnection,
     progress_bar: &ProgressBar,
     header: &Option<Header>,
     romfile_path: &P,
     rom: &Rom,
 ) -> SimpleResult<()> {
-    let (size, crc) = get_file_size_and_crc(progress_bar, romfile_path, &header, 1, 1).await?;
+    let (size, crc) =
+        get_file_size_and_crc(connection, progress_bar, romfile_path, header, 1, 1).await?;
     if i64::try_from(size).unwrap() != rom.size || crc != rom.crc {
         bail!("CRC or size mismatch");
     };
