@@ -57,9 +57,7 @@ pub async fn establish_connection(url: &str) -> SqlitePool {
     pool
 }
 
-pub async fn begin_transaction(
-    connection: &mut SqliteConnection,
-) -> Transaction<'_, Sqlite> {
+pub async fn begin_transaction(connection: &mut SqliteConnection) -> Transaction<'_, Sqlite> {
     Acquire::begin(connection)
         .await
         .expect("Failed to begin transaction")
@@ -91,17 +89,22 @@ pub async fn close_connection(pool: &SqlitePool) {
     .expect("Failed to optimize the database");
 }
 
-pub async fn create_system(connection: &mut SqliteConnection, system_xml: &SystemXml) -> i64 {
+pub async fn create_system(
+    connection: &mut SqliteConnection,
+    system_xml: &SystemXml,
+    arcade: bool,
+) -> i64 {
     let name = system_xml.name.replace(" (Parent-Clone)", "");
     sqlx::query!(
         "
-        INSERT INTO systems (name, description, version, url)
-        VALUES (?, ?, ?, ?)
+        INSERT INTO systems (name, description, version, url, arcade)
+        VALUES (?, ?, ?, ?, ?)
         ",
         name,
         system_xml.description,
         system_xml.version,
         system_xml.url,
+        arcade,
     )
     .execute(connection)
     .await
@@ -109,18 +112,24 @@ pub async fn create_system(connection: &mut SqliteConnection, system_xml: &Syste
     .last_insert_rowid()
 }
 
-pub async fn update_system(connection: &mut SqliteConnection, id: i64, system_xml: &SystemXml) {
+pub async fn update_system(
+    connection: &mut SqliteConnection,
+    id: i64,
+    system_xml: &SystemXml,
+    arcade: bool,
+) {
     let name = system_xml.name.replace(" (Parent-Clone)", "");
     sqlx::query!(
         "
         UPDATE systems
-        SET name = ?, description = ?, version = ?, url = ?
+        SET name = ?, description = ?, version = ?, url = ?, arcade = ?
         WHERE id = ?
         ",
         name,
         system_xml.description,
         system_xml.version,
         system_xml.url,
+        arcade,
         id,
     )
     .execute(connection)
@@ -192,6 +201,22 @@ pub async fn update_systems_mark_incomplete(connection: &mut SqliteConnection) {
     .unwrap_or_else(|_| panic!("Error while marking systems as incomplete"));
 }
 
+pub async fn update_system_merging(connection: &mut SqliteConnection, id: i64, merging: Merging) {
+    let merging = merging as i8;
+    sqlx::query!(
+        "
+        UPDATE systems
+        SET merging = ?
+        WHERE id = ?
+        ",
+        merging,
+        id,
+    )
+    .execute(connection)
+    .await
+    .unwrap_or_else(|_| panic!("Error while updating system with id {} merging", merging));
+}
+
 pub async fn find_systems(connection: &mut SqliteConnection) -> Vec<System> {
     sqlx::query_as!(
         System,
@@ -206,6 +231,21 @@ pub async fn find_systems(connection: &mut SqliteConnection) -> Vec<System> {
     .expect("Error while finding systems")
 }
 
+pub async fn find_arcade_systems(connection: &mut SqliteConnection) -> Vec<System> {
+    sqlx::query_as!(
+        System,
+        "
+        SELECT *
+        FROM systems
+        WHERE arcade = true
+        ORDER BY name
+        ",
+    )
+    .fetch_all(connection)
+    .await
+    .expect("Error while finding arcade systems")
+}
+
 pub async fn find_systems_by_url(connection: &mut SqliteConnection, url: &str) -> Vec<System> {
     sqlx::query_as!(
         System,
@@ -219,7 +259,7 @@ pub async fn find_systems_by_url(connection: &mut SqliteConnection, url: &str) -
     )
     .fetch_all(connection)
     .await
-    .expect("Error while finding system names")
+    .unwrap_or_else(|_| panic!("Error while finding systems with url {}", url))
 }
 
 pub async fn find_system_by_id(connection: &mut SqliteConnection, id: i64) -> System {
@@ -277,17 +317,22 @@ pub async fn create_game(
     regions: &str,
     system_id: i64,
     parent_id: Option<i64>,
+    bios_id: Option<i64>,
 ) -> i64 {
+    let bios = game_xml.isbios.is_some() && game_xml.isbios.as_ref().unwrap() == "yes";
     sqlx::query!(
         "
-        INSERT INTO games (name, description, regions, system_id, parent_id)
-        VALUES (?, ?, ?, ?, ?)
+        INSERT INTO games (name, description, comment, bios, regions, system_id, parent_id, bios_id)
+        VALUES (?, ?, ?, ?, ?, ?, ?, ?)
         ",
         game_xml.name,
         game_xml.description,
+        game_xml.comment,
+        bios,
         regions,
         system_id,
         parent_id,
+        bios_id,
     )
     .execute(connection)
     .await
@@ -302,18 +347,23 @@ pub async fn update_game(
     regions: &str,
     system_id: i64,
     parent_id: Option<i64>,
+    bios_id: Option<i64>,
 ) {
+    let bios = game_xml.isbios.is_some() && game_xml.isbios.as_ref().unwrap() == "yes";
     sqlx::query!(
         "
         UPDATE games
-        SET name = ?, description = ?, regions = ?, system_id = ?, parent_id = ?
+        SET name = ?, description = ?, comment = ?, bios = ?, regions = ?, system_id = ?, parent_id = ?, bios_id = ?
         WHERE id = ?
         ",
         game_xml.name,
         game_xml.description,
+        game_xml.comment,
+        bios,
         regions,
         system_id,
         parent_id,
+        bios_id,
         id,
     )
     .execute(connection)
@@ -335,7 +385,8 @@ pub async fn update_games_by_system_id_mark_complete(
             SELECT r.id
             FROM roms r
             WHERE r.game_id = games.id
-            AND r.romfile_id IS null
+            AND r.romfile_id IS NULL
+            AND r.parent_id IS NULL
         )
         ",
         system_id,
@@ -359,7 +410,8 @@ pub async fn update_games_by_system_id_mark_incomplete(
             SELECT r.id
             FROM roms r
             WHERE r.game_id = games.id
-            AND r.romfile_id IS null
+            AND r.romfile_id IS NULL
+            AND r.parent_id IS NULL
         )
         ",
         system_id,
@@ -379,7 +431,8 @@ pub async fn update_games_mark_incomplete(connection: &mut SqliteConnection) {
             SELECT r.id
             FROM roms r
             WHERE r.game_id = games.id
-            AND r.romfile_id IS null
+            AND r.parent_id IS NULL
+            AND r.romfile_id IS NULL
         )
         ",
     )
@@ -393,14 +446,17 @@ pub async fn update_games_sorting(
     ids: &[i64],
     sorting: Sorting,
 ) -> u64 {
+    let sorting = sorting as i8;
     let sql = format!(
         "
         UPDATE games
         SET sorting = {}
         WHERE id IN ({})
+        AND sorting != {}
         ",
-        sorting as i8,
-        ids.iter().join(",")
+        sorting,
+        ids.iter().join(","),
+        sorting,
     );
     sqlx::query(&sql)
         .execute(connection)
@@ -413,7 +469,7 @@ pub async fn find_games(connection: &mut SqliteConnection) -> Vec<Game> {
     sqlx::query_as!(
         Game,
         "
-        SELECT id, name, description, regions, sorting as \"sorting: _\", complete, system_id, parent_id
+        SELECT *
         FROM games
         ORDER BY name
         ",
@@ -430,7 +486,7 @@ pub async fn find_games_by_system_id(
     sqlx::query_as!(
         Game,
         "
-        SELECT id, name, description, regions, sorting as \"sorting: _\", complete, system_id, parent_id
+        SELECT *
         FROM games
         WHERE system_id = ?
         ORDER BY name
@@ -442,6 +498,32 @@ pub async fn find_games_by_system_id(
     .unwrap_or_else(|_| panic!("Error while finding games with system id {}", system_id))
 }
 
+pub async fn find_incomplete_games_by_system_id(
+    connection: &mut SqliteConnection,
+    system_id: i64,
+) -> Vec<Game> {
+    sqlx::query_as!(
+        Game,
+        "
+        SELECT *
+        FROM games
+        WHERE system_id = ?
+        AND complete = false
+        AND sorting != 2
+        ORDER BY name
+        ",
+        system_id,
+    )
+    .fetch_all(connection)
+    .await
+    .unwrap_or_else(|_| {
+        panic!(
+            "Error while finding incomplete games with system id {}",
+            system_id
+        )
+    })
+}
+
 pub async fn find_games_by_ids(connection: &mut SqliteConnection, ids: &[i64]) -> Vec<Game> {
     let sql = format!(
         "
@@ -465,7 +547,7 @@ pub async fn find_parent_games_by_system_id(
     sqlx::query_as!(
         Game,
         "
-        SELECT id, name, description, regions, sorting as \"sorting: _\", complete, system_id, parent_id
+        SELECT *
         FROM games
         WHERE system_id = ?
         AND parent_id IS NULL
@@ -490,7 +572,7 @@ pub async fn find_clone_games_by_system_id(
     sqlx::query_as!(
         Game,
         "
-        SELECT id, name, description, regions, sorting as \"sorting: _\", complete, system_id, parent_id
+        SELECT *
         FROM games
         WHERE system_id = ?
         AND parent_id IS NOT NULL
@@ -512,7 +594,7 @@ pub async fn find_game_by_id(connection: &mut SqliteConnection, id: i64) -> Game
     sqlx::query_as!(
         Game,
         "
-        SELECT id, name, description, regions, sorting as \"sorting: _\", complete, system_id, parent_id
+        SELECT *
         FROM games
         WHERE id = ?
         ",
@@ -523,20 +605,23 @@ pub async fn find_game_by_id(connection: &mut SqliteConnection, id: i64) -> Game
     .unwrap_or_else(|_| panic!("Error while finding game with id {}", id))
 }
 
-pub async fn find_game_by_name_and_system_id(
+pub async fn find_game_by_name_and_bios_and_system_id(
     connection: &mut SqliteConnection,
     name: &str,
+    bios: bool,
     system_id: i64,
 ) -> Option<Game> {
     sqlx::query_as!(
         Game,
         "
-        SELECT id, name, description, regions, sorting as \"sorting: _\", complete, system_id, parent_id
+        SELECT *
         FROM games
         WHERE name = ?
+        AND bios = ?
         AND system_id = ?
         ",
         name,
+        bios,
         system_id,
     )
     .fetch_optional(connection)
@@ -549,6 +634,57 @@ pub async fn find_game_by_name_and_system_id(
     })
 }
 
+pub async fn find_games_with_romfiles_by_system_id(
+    connection: &mut SqliteConnection,
+    system_id: i64,
+) -> Vec<Game> {
+    sqlx::query_as!(
+        Game,
+        "
+        SELECT *
+        FROM games
+        WHERE system_id = ?
+        AND id IN (
+            SELECT DISTINCT(game_id)
+            FROM roms
+            WHERE romfile_id IS NOT NULL
+        )
+        ORDER BY name
+        ",
+        system_id,
+    )
+    .fetch_all(connection)
+    .await
+    .expect("Error while finding games with romfiles")
+}
+
+pub async fn find_games_with_romfiles_by_name_and_system_id(
+    connection: &mut SqliteConnection,
+    name: &str,
+    system_id: i64,
+) -> Vec<Game> {
+    sqlx::query_as!(
+        Game,
+        "
+        SELECT *
+        FROM games
+        WHERE name LIKE ?
+        AND system_id = ?
+        AND id IN (
+            SELECT DISTINCT(game_id)
+            FROM roms
+            WHERE romfile_id IS NOT NULL
+        )
+        ORDER BY name
+        ",
+        name,
+        system_id,
+    )
+    .fetch_all(connection)
+    .await
+    .expect("Error while finding games with romfiles")
+}
+
 pub async fn delete_game_by_name_and_system_id(
     connection: &mut SqliteConnection,
     name: &str,
@@ -573,22 +709,30 @@ pub async fn delete_game_by_name_and_system_id(
     });
 }
 
-pub async fn create_rom(connection: &mut SqliteConnection, rom_xml: &RomXml, game_id: i64) -> i64 {
-    let crc = rom_xml.crc.to_lowercase();
-    let md5 = rom_xml.md5.to_lowercase();
-    let sha1 = rom_xml.sha1.to_lowercase();
+pub async fn create_rom(
+    connection: &mut SqliteConnection,
+    rom_xml: &RomXml,
+    bios: bool,
+    game_id: i64,
+    parent_id: Option<i64>,
+) -> i64 {
+    let crc = rom_xml.crc.as_ref().unwrap().to_lowercase();
+    let md5 = rom_xml.md5.as_ref().map(|md5| md5.to_lowercase());
+    let sha1 = rom_xml.sha1.as_ref().map(|sha1| sha1.to_lowercase());
     sqlx::query!(
         "
-        INSERT INTO roms (name, size, crc, md5, sha1, rom_status, game_id)
-        VALUES (?, ?, ?, ?, ?, ?, ?)
+        INSERT INTO roms (name, bios, size, crc, md5, sha1, rom_status, game_id, parent_id)
+        VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)
         ",
         rom_xml.name,
+        bios,
         rom_xml.size,
         crc,
         md5,
         sha1,
         rom_xml.status,
         game_id,
+        parent_id,
     )
     .execute(connection)
     .await
@@ -600,24 +744,28 @@ pub async fn update_rom(
     connection: &mut SqliteConnection,
     id: i64,
     rom_xml: &RomXml,
+    bios: bool,
     game_id: i64,
+    parent_id: Option<i64>,
 ) {
-    let crc = rom_xml.crc.to_lowercase();
-    let md5 = rom_xml.md5.to_lowercase();
-    let sha1 = rom_xml.sha1.to_lowercase();
+    let crc = rom_xml.crc.as_ref().unwrap().to_lowercase();
+    let md5 = rom_xml.md5.as_ref().map(|md5| md5.to_lowercase());
+    let sha1 = rom_xml.sha1.as_ref().map(|sha1| sha1.to_lowercase());
     sqlx::query!(
         "
         UPDATE roms
-        SET name = ?, size = ?, crc = ?, md5 = ?, sha1 = ?, rom_status = ?, game_id = ?
+        SET name = ?, bios = ?, size = ?, crc = ?, md5 = ?, sha1 = ?, rom_status = ?, game_id = ?, parent_id = ?
         WHERE id = ?
         ",
         rom_xml.name,
+        bios,
         rom_xml.size,
         crc,
         md5,
         sha1,
         rom_xml.status,
         game_id,
+        parent_id,
         id,
     )
     .execute(connection)
@@ -658,7 +806,74 @@ pub async fn find_roms(connection: &mut SqliteConnection) -> Vec<Rom> {
     .expect("Error while finding roms")
 }
 
-pub async fn find_roms_by_game_id(connection: &mut SqliteConnection, game_id: i64) -> Vec<Rom> {
+pub async fn find_roms_by_romfile_id(
+    connection: &mut SqliteConnection,
+    romfile_id: i64,
+) -> Vec<Rom> {
+    sqlx::query_as!(
+        Rom,
+        "
+        SELECT *
+        FROM roms
+        WHERE romfile_id = ?
+        ",
+        romfile_id,
+    )
+    .fetch_all(connection)
+    .await
+    .unwrap_or_else(|_| panic!("Error while finding roms with romfile id {}", romfile_id))
+}
+
+pub async fn find_roms_by_game_id_no_parents(
+    connection: &mut SqliteConnection,
+    game_id: i64,
+) -> Vec<Rom> {
+    sqlx::query_as!(
+        Rom,
+        "
+        SELECT *
+        FROM roms
+        WHERE game_id = ?
+        AND parent_id IS NULL
+        ORDER BY name
+        ",
+        game_id,
+    )
+    .fetch_all(connection)
+    .await
+    .unwrap_or_else(|_| panic!("Error while finding roms with game id {}", game_id))
+}
+
+pub async fn find_roms_by_game_id_parents_no_parent_bioses(
+    connection: &mut SqliteConnection,
+    game_id: i64,
+) -> Vec<Rom> {
+    sqlx::query_as!(
+        Rom,
+        "
+        SELECT *
+        FROM roms
+        WHERE game_id = ?
+        AND (
+            parent_id IS NULL
+            OR (
+                parent_id IS NOT NULL
+                AND bios = false
+            )
+        )
+        ORDER BY name
+        ",
+        game_id,
+    )
+    .fetch_all(connection)
+    .await
+    .unwrap_or_else(|_| panic!("Error while finding roms with game id {}", game_id))
+}
+
+pub async fn find_roms_by_game_id_parents(
+    connection: &mut SqliteConnection,
+    game_id: i64,
+) -> Vec<Rom> {
     sqlx::query_as!(
         Rom,
         "
@@ -671,7 +886,48 @@ pub async fn find_roms_by_game_id(connection: &mut SqliteConnection, game_id: i6
     )
     .fetch_all(connection)
     .await
-    .unwrap_or_else(|_| panic!("Error while finding rom with game id {}", game_id))
+    .unwrap_or_else(|_| panic!("Error while finding roms with game id {}", game_id))
+}
+
+pub async fn find_roms_by_game_id_parents_only(
+    connection: &mut SqliteConnection,
+    game_id: i64,
+) -> Vec<Rom> {
+    sqlx::query_as!(
+        Rom,
+        "
+        SELECT *
+        FROM roms
+        WHERE game_id = ?
+        AND parent_id IS NOT NULL
+        ORDER BY name
+        ",
+        game_id,
+    )
+    .fetch_all(connection)
+    .await
+    .unwrap_or_else(|_| panic!("Error while finding roms with game id {}", game_id))
+}
+
+pub async fn find_roms_by_game_id_parent_bioses_only(
+    connection: &mut SqliteConnection,
+    game_id: i64,
+) -> Vec<Rom> {
+    sqlx::query_as!(
+        Rom,
+        "
+        SELECT *
+        FROM roms
+        WHERE game_id = ?
+        AND parent_id IS NOT NULL
+        AND bios = true
+        ORDER BY name
+        ",
+        game_id,
+    )
+    .fetch_all(connection)
+    .await
+    .unwrap_or_else(|_| panic!("Error while finding roms with game id {}", game_id))
 }
 
 pub async fn find_roms_without_romfile_by_game_ids(
@@ -680,12 +936,13 @@ pub async fn find_roms_without_romfile_by_game_ids(
 ) -> Vec<Rom> {
     let sql = format!(
         "
-    SELECT *
-    FROM roms
-    WHERE romfile_id IS NULL
-    AND game_id IN ({})
-    ORDER BY name
-    ",
+        SELECT *
+        FROM roms
+        WHERE romfile_id IS NULL
+        AND parent_id IS NULL
+        AND game_id IN ({})
+        ORDER BY name
+        ",
         game_ids.iter().join(", ")
     );
     sqlx::query_as::<_, Rom>(&sql)
@@ -694,6 +951,47 @@ pub async fn find_roms_without_romfile_by_game_ids(
         .expect("Error while finding roms with romfile")
 }
 
+pub async fn find_roms_without_romfile_by_game_id_split(
+    connection: &mut SqliteConnection,
+    game_id: i64,
+) -> Vec<Rom> {
+    sqlx::query_as!(
+        Rom,
+        "
+        SELECT *
+        FROM roms
+        WHERE romfile_id IS NULL
+        AND parent_id IS NULL
+        AND game_id = ?
+        ORDER BY name
+        ",
+        game_id
+    )
+    .fetch_all(connection)
+    .await
+    .expect("Error while finding roms with romfile")
+}
+
+pub async fn find_roms_without_romfile_by_game_id_non_merged(
+    connection: &mut SqliteConnection,
+    game_id: i64,
+) -> Vec<Rom> {
+    sqlx::query_as!(
+        Rom,
+        "
+        SELECT *
+        FROM roms
+        WHERE romfile_id IS NULL
+        AND game_id = ?
+        ORDER BY name
+        ",
+        game_id
+    )
+    .fetch_all(connection)
+    .await
+    .expect("Error while finding roms with romfile")
+}
+
 pub async fn find_roms_with_romfile_by_system_id(
     connection: &mut SqliteConnection,
     system_id: i64,
@@ -701,15 +999,12 @@ pub async fn find_roms_with_romfile_by_system_id(
     sqlx::query_as!(
         Rom,
         "
-        SELECT *
-        FROM roms
-        WHERE romfile_id IS NOT NULL
-        AND game_id IN (
-            SELECT id
-            FROM games
-            WHERE system_id = ?
-        )
-        ORDER BY name
+        SELECT r.id, r.name, r.bios, r.size, r.crc, r.md5, r.sha1, r.rom_status, r.game_id, r.romfile_id, r.parent_id
+        FROM roms AS r
+        JOIN games AS g ON r.game_id = g.id
+        WHERE r.romfile_id IS NOT NULL
+        AND g.system_id = ?
+        ORDER BY r.name
         ",
         system_id,
     )
@@ -718,32 +1013,6 @@ pub async fn find_roms_with_romfile_by_system_id(
     .expect("Error while finding roms with romfile")
 }
 
-pub async fn find_roms_with_romfile_by_system_id_and_name(
-    connection: &mut SqliteConnection,
-    system_id: i64,
-    name: &str,
-) -> Vec<Rom> {
-    let sql = format!(
-        "
-        SELECT *
-        FROM roms
-        WHERE romfile_id IS NOT NULL
-        AND game_id IN (
-            SELECT id
-            FROM games
-            WHERE system_id = {}
-            AND name LIKE '%{}%'
-        )
-        ORDER BY name
-    ",
-        system_id, name
-    );
-    sqlx::query_as::<_, Rom>(&sql)
-        .fetch_all(connection)
-        .await
-        .expect("Error while finding roms with romfile")
-}
-
 pub async fn find_roms_with_romfile_by_game_ids(
     connection: &mut SqliteConnection,
     game_ids: &[i64],
@@ -764,6 +1033,69 @@ pub async fn find_roms_with_romfile_by_game_ids(
         .expect("Error while finding roms with romfile")
 }
 
+pub async fn find_roms_with_romfile_by_size_and_crc_and_system_id(
+    connection: &mut SqliteConnection,
+    size: i64,
+    crc: &str,
+    system_id: i64,
+) -> Vec<Rom> {
+    sqlx::query_as!(
+        Rom,
+        "
+        SELECT r.id, r.name, r.bios, r.size, r.crc, r.md5, r.sha1, r.rom_status, r.game_id, r.romfile_id, r.parent_id
+        FROM roms AS r
+        JOIN games AS g ON r.game_id = g.id
+        WHERE r.romfile_id IS NOT NULL
+        AND r.size = ?
+        AND r.crc = ?
+        AND r.parent_id IS NULL
+        AND g.system_id = ?
+        ORDER BY r.name
+        ",
+        size,
+        crc,
+        system_id
+    )
+    .fetch_all(connection)
+    .await
+    .expect("Error while finding roms with romfile")
+}
+
+pub async fn find_roms_without_romfile_by_size_and_crc_and_system_id(
+    connection: &mut SqliteConnection,
+    size: u64,
+    crc: &str,
+    system_id: i64,
+) -> Vec<Rom> {
+    let size = i64::try_from(size).unwrap();
+    let crc = crc.to_lowercase();
+    sqlx::query_as!(
+        Rom,
+        "
+        SELECT r.id, r.name, r.bios, r.size, r.crc, r.md5, r.sha1, r.rom_status, r.game_id, r.romfile_id, r.parent_id
+        FROM roms AS r
+        JOIN games AS g ON r.game_id = g.id
+        WHERE r.romfile_id IS NULL
+        AND r.size = ?
+        AND r.crc = ?
+        AND r.parent_id IS NULL
+        AND g.system_id = ?
+        ORDER BY r.name
+        ",
+        size,
+        crc,
+        system_id,
+    )
+    .fetch_all(connection)
+    .await
+    .unwrap_or_else(|_| {
+        panic!(
+            "Error while finding rom with size {} and CRC {} and system id {}",
+            size, crc, system_id
+        )
+    })
+}
+
 pub async fn find_rom_by_name_and_game_id(
     connection: &mut SqliteConnection,
     name: &str,
@@ -790,35 +1122,32 @@ pub async fn find_rom_by_name_and_game_id(
     })
 }
 
-pub async fn find_roms_by_size_and_crc_and_system_id(
+pub async fn find_rom_by_size_and_crc_and_game_id(
     connection: &mut SqliteConnection,
-    size: u64,
+    size: i64,
     crc: &str,
-    system_id: i64,
-) -> Vec<Rom> {
-    let size = i64::try_from(size).unwrap();
+    game_id: i64,
+) -> Option<Rom> {
     let crc = crc.to_lowercase();
     sqlx::query_as!(
         Rom,
         "
-        SELECT r.id, r.name, r.size, r.crc, r.md5, r.sha1, r.rom_status, r.game_id, r.romfile_id
-        FROM roms AS r
-        JOIN games AS g ON r.game_id = g.id
-        WHERE r.size = ?
-        AND r.crc = ?
-        AND g.system_id = ?
-        ORDER BY r.name
+        SELECT *
+        FROM roms
+        WHERE size = ?
+        AND crc = ?
+        AND game_id = ?
         ",
         size,
         crc,
-        system_id,
+        game_id,
     )
-    .fetch_all(connection)
+    .fetch_optional(connection)
     .await
     .unwrap_or_else(|_| {
         panic!(
-            "Error while finding rom with size {} and CRC {} and system id {}",
-            size, crc, system_id
+            "Error while finding rom with size {} and CRC {} and game id {}",
+            size, crc, game_id
         )
     })
 }
@@ -919,7 +1248,7 @@ pub async fn find_romfiles_by_system_id(
         SELECT *
         FROM romfiles
         WHERE id IN (
-            SELECT romfile_id
+            SELECT DISTINCT(romfile_id)
             FROM roms
             WHERE game_id IN (
                 SELECT id
@@ -1018,18 +1347,15 @@ pub async fn create_header(
     detector_xml: &DetectorXml,
     system_id: i64,
 ) -> i64 {
-    let start_byte = i64::from_str_radix(&detector_xml.rule.data.offset, 16).unwrap();
     let size = i64::from_str_radix(&detector_xml.rule.start_offset, 16).unwrap();
     sqlx::query!(
         "
-        INSERT INTO headers (name, version, start_byte, size, hex_value, system_id)
-        VALUES (?, ?, ?, ?, ?, ?)
+        INSERT INTO headers (name, version, size, system_id)
+        VALUES (?, ?, ?, ?)
         ",
         detector_xml.name,
         detector_xml.version,
-        start_byte,
         size,
-        detector_xml.rule.data.value,
         system_id,
     )
     .execute(connection)
@@ -1044,19 +1370,16 @@ pub async fn update_header(
     detector_xml: &DetectorXml,
     system_id: i64,
 ) {
-    let start_byte = i64::from_str_radix(&detector_xml.rule.data.offset, 16).unwrap();
     let size = i64::from_str_radix(&detector_xml.rule.start_offset, 16).unwrap();
     sqlx::query!(
         "
         UPDATE headers
-        SET name = ?, version = ?, start_byte = ?, size = ?, hex_value = ?, system_id = ?
+        SET name = ?, version = ?, size = ?, system_id = ?
         WHERE id = ?
         ",
         detector_xml.name,
         detector_xml.version,
-        start_byte,
         size,
-        detector_xml.rule.data.value,
         system_id,
         id,
     )
@@ -1083,6 +1406,58 @@ pub async fn find_header_by_system_id(
     .unwrap_or_else(|_| panic!("Error while finding header with system id {}", system_id))
 }
 
+pub async fn create_rule(
+    connection: &mut SqliteConnection,
+    data_xml: &DataXml,
+    header_id: i64,
+) -> i64 {
+    let start_byte = i64::from_str_radix(&data_xml.offset, 16).unwrap();
+    sqlx::query!(
+        "
+        INSERT INTO rules (start_byte, hex_value, header_id)
+        VALUES (?, ?, ?)
+        ",
+        start_byte,
+        data_xml.value,
+        header_id,
+    )
+    .execute(connection)
+    .await
+    .expect("Error while creating rule")
+    .last_insert_rowid()
+}
+
+pub async fn find_rules_by_header_id(
+    connection: &mut SqliteConnection,
+    header_id: i64,
+) -> Vec<Rule> {
+    sqlx::query_as!(
+        Rule,
+        "
+        SELECT *
+        FROM rules
+        WHERE header_id = ?
+        ",
+        header_id,
+    )
+    .fetch_all(connection)
+    .await
+    .unwrap_or_else(|_| panic!("Error while finding rules with header id {}", header_id))
+}
+
+pub async fn delete_rules_by_header_id(connection: &mut SqliteConnection, header_id: i64) {
+    sqlx::query!(
+        "
+        DELETE FROM rules
+        WHERE header_id = ?
+        ",
+        header_id,
+    )
+    .execute(connection)
+    .await
+    .unwrap_or_else(|_| panic!("Error while deleting rules with header_id {}", header_id));
+}
+
 pub async fn create_setting(connection: &mut SqliteConnection, key: &str, value: Option<String>) {
     sqlx::query!(
         "
@@ -1140,16 +1515,3 @@ pub async fn find_setting_by_key(connection: &mut SqliteConnection, key: &str) -
     .await
     .unwrap_or_else(|_| panic!("Error while finding setting with key {}", key))
 }
-
-pub async fn delete_setting_by_key(connection: &mut SqliteConnection, key: &str) {
-    sqlx::query!(
-        "
-        DELETE FROM settings
-        WHERE key = ?
-        ",
-        key,
-    )
-    .execute(connection)
-    .await
-    .unwrap_or_else(|_| panic!("Error while deleting setting with key {}", key));
-}
