@@ -5,57 +5,72 @@ use super::progress::*;
 use super::util::*;
 use super::SimpleResult;
 use async_std::path::Path;
-use clap::{App, Arg, ArgMatches, SubCommand};
+use clap::{App, Arg, ArgMatches};
 use indicatif::ProgressBar;
 use quick_xml::de;
 use rayon::prelude::*;
 use regex::Regex;
+use rust_embed::RustEmbed;
 use shiratsu_naming::naming::nointro::{NoIntroName, NoIntroToken};
 use shiratsu_naming::naming::TokenizedName;
 use shiratsu_naming::region::Region;
 use sqlx::sqlite::SqliteConnection;
 use std::io;
+use std::str;
+use vec_drain_where::VecDrainWhereExt;
 
 lazy_static! {
     pub static ref SYSTEM_NAME_REGEX: Regex = Regex::new(r"\(.*\)").unwrap();
 }
 
-pub fn subcommand<'a, 'b>() -> App<'a, 'b> {
-    SubCommand::with_name("import-dats")
-        .about("Parses and imports No-Intro and Redump DAT files into oxyromon")
+#[derive(RustEmbed)]
+#[folder = "data/"]
+struct Assets;
+
+pub fn subcommand<'a>() -> App<'a> {
+    App::new("import-dats")
+        .about("Parse and import Logiqx DAT files into oxyromon")
         .arg(
-            Arg::with_name("DATS")
-                .help("Sets the DAT files to import")
+            Arg::new("DATS")
+                .help("Set the DAT files to import")
                 .required(true)
-                .multiple(true)
-                .index(1),
+                .multiple_values(true)
+                .index(1)
+                .allow_invalid_utf8(true),
         )
         .arg(
-            Arg::with_name("INFO")
-                .short("i")
+            Arg::new("INFO")
+                .short('i')
                 .long("info")
-                .help("Shows the DAT information and exit")
+                .help("Show the DAT information and exit")
                 .required(false),
         )
         .arg(
-            Arg::with_name("SKIP_HEADER")
-                .short("s")
+            Arg::new("SKIP_HEADER")
+                .short('s')
                 .long("skip-header")
-                .help("Skips parsing the header even if the system has one")
+                .help("Skip parsing the header even if the system has one")
                 .required(false),
         )
         .arg(
-            Arg::with_name("FORCE")
-                .short("f")
+            Arg::new("FORCE")
+                .short('f')
                 .long("force")
-                .help("Forces import of outdated DAT files")
+                .help("Force import of outdated DAT files")
+                .required(false),
+        )
+        .arg(
+            Arg::new("ARCADE")
+                .short('a')
+                .long("arcade")
+                .help("Enable arcade mode")
                 .required(false),
         )
 }
 
 pub async fn main(
     connection: &mut SqliteConnection,
-    matches: &ArgMatches<'_>,
+    matches: &ArgMatches,
     progress_bar: &ProgressBar,
 ) -> SimpleResult<()> {
     let dat_paths: Vec<String> = matches.values_of_lossy("DATS").unwrap();
@@ -66,13 +81,15 @@ pub async fn main(
             progress_bar,
             &get_canonicalized_path(&dat_path).await?,
             matches.is_present("SKIP_HEADER"),
-        )?;
+        )
+        .await?;
         if !matches.is_present("INFO") {
             import_dat(
                 connection,
                 progress_bar,
                 &datfile_xml,
                 &detector_xml,
+                matches.is_present("ARCADE"),
                 matches.is_present("FORCE"),
             )
             .await?;
@@ -83,7 +100,7 @@ pub async fn main(
     Ok(())
 }
 
-pub fn parse_dat<P: AsRef<Path>>(
+pub async fn parse_dat<P: AsRef<Path>>(
     progress_bar: &ProgressBar,
     dat_path: &P,
     skip_header: bool,
@@ -102,11 +119,18 @@ pub fn parse_dat<P: AsRef<Path>>(
     if !skip_header {
         if let Some(clr_mame_pro_xml) = &datfile_xml.system.clrmamepro {
             progress_bar.println("Processing header");
-            let header_file_name = &clr_mame_pro_xml.header;
-            let header_file_path = dat_path.as_ref().parent().unwrap().join(header_file_name);
-            let header_file = open_file_sync(&header_file_path.as_path())?;
-            let reader = io::BufReader::new(header_file);
-            detector_xml = de::from_reader(reader).expect("Failed to parse header file");
+            if let Some(header_file_name) = &clr_mame_pro_xml.header {
+                let header_file_path = dat_path.as_ref().parent().unwrap().join(header_file_name);
+                if header_file_path.is_file().await {
+                    let header_file = open_file_sync(&header_file_path.as_path())?;
+                    let reader = io::BufReader::new(header_file);
+                    detector_xml = de::from_reader(reader).expect("Failed to parse header file");
+                } else {
+                    let header_file = Assets::get(header_file_name).unwrap();
+                    detector_xml = de::from_str(str::from_utf8(header_file.data.as_ref()).unwrap())
+                        .expect("Failed to parse header file");
+                }
+            }
         }
     };
 
@@ -118,6 +142,7 @@ pub async fn import_dat(
     progress_bar: &ProgressBar,
     datfile_xml: &DatfileXml,
     detector_xml: &Option<DetectorXml>,
+    arcade: bool,
     force: bool,
 ) -> SimpleResult<()> {
     progress_bar.println("Processing system");
@@ -125,17 +150,22 @@ pub async fn import_dat(
     let mut transaction = begin_transaction(connection).await;
 
     // persist system
-    let system_id =
-        match create_or_update_system(&mut transaction, progress_bar, &datfile_xml.system, force)
-            .await
-        {
-            Some(system_id) => system_id,
-            None => return Ok(()),
-        };
+    let system_id = match create_or_update_system(
+        &mut transaction,
+        progress_bar,
+        &datfile_xml.system,
+        arcade,
+        force,
+    )
+    .await
+    {
+        Some(system_id) => system_id,
+        None => return Ok(()),
+    };
 
     // persist header
     if let Some(detector_xml) = detector_xml {
-        create_or_update_header(&mut transaction, &detector_xml, system_id).await;
+        create_or_update_header(&mut transaction, detector_xml, system_id).await;
     }
 
     progress_bar.reset();
@@ -153,7 +183,8 @@ pub async fn import_dat(
             &mut transaction,
             &datfile_xml.games,
             system_id,
-            &progress_bar,
+            arcade,
+            progress_bar,
         )
         .await?,
     );
@@ -204,6 +235,7 @@ async fn create_or_update_system(
     connection: &mut SqliteConnection,
     progress_bar: &ProgressBar,
     system_xml: &SystemXml,
+    arcade: bool,
     force: bool,
 ) -> Option<i64> {
     let mut system = find_system_by_name(connection, &system_xml.name).await;
@@ -218,13 +250,13 @@ async fn create_or_update_system(
     match system {
         Some(system) => {
             if is_update(progress_bar, &system.version, &system_xml.version) || force {
-                update_system(connection, system.id, system_xml).await;
+                update_system(connection, system.id, system_xml, arcade).await;
                 Some(system.id)
             } else {
                 None
             }
         }
-        None => Some(create_system(connection, system_xml).await),
+        None => Some(create_system(connection, system_xml, arcade).await),
     }
 }
 
@@ -234,124 +266,207 @@ async fn create_or_update_header(
     system_id: i64,
 ) {
     let header = find_header_by_system_id(connection, system_id).await;
-    match header {
+    let header_id = match header {
         Some(header) => {
             update_header(connection, header.id, detector_xml, system_id).await;
+            delete_rules_by_header_id(connection, header.id).await;
             header.id
         }
         None => create_header(connection, detector_xml, system_id).await,
     };
+    for data_xml in &detector_xml.rule.data {
+        create_rule(connection, data_xml, header_id).await;
+    }
 }
 
 async fn create_or_update_games(
     connection: &mut SqliteConnection,
     games_xml: &[GameXml],
     system_id: i64,
+    arcade: bool,
     progress_bar: &ProgressBar,
 ) -> SimpleResult<Vec<i64>> {
     let mut orphan_romfile_ids: Vec<i64> = Vec::new();
-    let parent_games_xml: Vec<&GameXml> = games_xml
-        .iter()
-        .filter(|game_xml| game_xml.cloneof.is_none())
-        .collect();
-    let child_games_xml: Vec<&GameXml> = games_xml
-        .iter()
-        .filter(|game_xml| game_xml.cloneof.is_some())
-        .collect();
-    for parent_game_xml in parent_games_xml {
-        let game =
-            find_game_by_name_and_system_id(connection, &parent_game_xml.name, system_id).await;
-        let game_id = match game {
-            Some(game) => {
-                update_game(
-                    connection,
-                    game.id,
-                    parent_game_xml,
-                    &get_regions_from_game_name(&parent_game_xml.name).unwrap(),
-                    system_id,
-                    None,
-                )
-                .await;
-                game.id
-            }
-            None => {
-                create_game(
-                    connection,
-                    parent_game_xml,
-                    &get_regions_from_game_name(&parent_game_xml.name).unwrap(),
-                    system_id,
-                    None,
-                )
-                .await
-            }
-        };
-        if !parent_game_xml.roms.is_empty() {
-            orphan_romfile_ids.append(
-                &mut create_or_update_roms(connection, &parent_game_xml.roms, game_id).await,
-            );
-        }
-        orphan_romfile_ids
-            .append(&mut delete_old_roms(connection, &parent_game_xml.roms, game_id).await);
-        progress_bar.inc(1)
-    }
-    for child_game_xml in child_games_xml {
-        let game =
-            find_game_by_name_and_system_id(connection, &child_game_xml.name, system_id).await;
-        let parent_game = find_game_by_name_and_system_id(
+    let (mut parent_games_xml, mut child_games_xml): (Vec<&GameXml>, Vec<&GameXml>) = games_xml
+        .par_iter()
+        .partition(|game_xml| game_xml.cloneof.is_none() && game_xml.romof.is_none());
+    for game_xml in &parent_games_xml {
+        let game = find_game_by_name_and_bios_and_system_id(
             connection,
-            child_game_xml.cloneof.as_ref().unwrap(),
+            &game_xml.name,
+            game_xml.isbios.is_some() && game_xml.isbios.as_ref().unwrap() == "yes",
             system_id,
         )
-        .await
-        .unwrap();
+        .await;
+        let regions = match arcade {
+            false => get_regions_from_game_name(&game_xml.name).unwrap(),
+            true => String::new(),
+        };
         let game_id = match game {
             Some(game) => {
                 update_game(
-                    connection,
-                    game.id,
-                    child_game_xml,
-                    &get_regions_from_game_name(&child_game_xml.name).unwrap(),
-                    system_id,
-                    Some(parent_game.id),
+                    connection, game.id, game_xml, &regions, system_id, None, None,
                 )
                 .await;
                 game.id
             }
-            None => {
-                create_game(
-                    connection,
-                    child_game_xml,
-                    &get_regions_from_game_name(&child_game_xml.name).unwrap(),
-                    system_id,
-                    Some(parent_game.id),
-                )
-                .await
-            }
+            None => create_game(connection, game_xml, &regions, system_id, None, None).await,
         };
-        if !child_game_xml.roms.is_empty() {
+        if !game_xml.roms.is_empty() {
             orphan_romfile_ids.append(
-                &mut create_or_update_roms(connection, &child_game_xml.roms, game_id).await,
+                &mut create_or_update_roms(
+                    connection,
+                    progress_bar,
+                    &game_xml.roms,
+                    game_xml.isbios.is_some() && game_xml.isbios.as_ref().unwrap() == "yes",
+                    game_id,
+                )
+                .await,
             );
         }
-        orphan_romfile_ids
-            .append(&mut delete_old_roms(connection, &child_game_xml.roms, game_id).await);
+        orphan_romfile_ids.append(&mut delete_old_roms(connection, &game_xml.roms, game_id).await);
         progress_bar.inc(1)
     }
+    while !child_games_xml.is_empty() {
+        let parent_game_names: Vec<&str> = parent_games_xml
+            .par_iter()
+            .map(|game_xml| game_xml.name.as_str())
+            .collect();
+        parent_games_xml = child_games_xml
+            .e_drain_where(|&mut child_game_xml| {
+                parent_game_names.contains(
+                    &child_game_xml
+                        .cloneof
+                        .as_ref()
+                        .or_else(|| child_game_xml.romof.as_ref())
+                        .unwrap()
+                        .as_str(),
+                )
+            })
+            .collect();
+        for game_xml in &parent_games_xml {
+            let game = find_game_by_name_and_bios_and_system_id(
+                connection,
+                &game_xml.name,
+                game_xml.isbios.is_some() && game_xml.isbios.as_ref().unwrap() == "yes",
+                system_id,
+            )
+            .await;
+            let parent_game = match game_xml.cloneof.as_ref() {
+                Some(name) => {
+                    find_game_by_name_and_bios_and_system_id(connection, name, false, system_id)
+                        .await
+                }
+                None => None,
+            };
+            let bios_game: Option<Game> = match game_xml.romof.as_ref() {
+                Some(name) => {
+                    find_game_by_name_and_bios_and_system_id(connection, name, true, system_id)
+                        .await
+                }
+                None => None,
+            };
+            let regions = match arcade {
+                false => get_regions_from_game_name(&game_xml.name).unwrap(),
+                true => String::new(),
+            };
+            let game_id = match game {
+                Some(game) => {
+                    update_game(
+                        connection,
+                        game.id,
+                        game_xml,
+                        &regions,
+                        system_id,
+                        parent_game.map(|game| game.id),
+                        bios_game.map(|game| game.id),
+                    )
+                    .await;
+                    game.id
+                }
+                None => {
+                    create_game(
+                        connection,
+                        game_xml,
+                        &regions,
+                        system_id,
+                        parent_game.map(|game| game.id),
+                        bios_game.map(|game| game.id),
+                    )
+                    .await
+                }
+            };
+            if !game_xml.roms.is_empty() {
+                orphan_romfile_ids.append(
+                    &mut create_or_update_roms(
+                        connection,
+                        progress_bar,
+                        &game_xml.roms,
+                        game_xml.isbios.is_some() && game_xml.isbios.as_ref().unwrap() == "yes",
+                        game_id,
+                    )
+                    .await,
+                );
+            }
+            orphan_romfile_ids
+                .append(&mut delete_old_roms(connection, &game_xml.roms, game_id).await);
+            progress_bar.inc(1)
+        }
+    }
     Ok(orphan_romfile_ids)
 }
 
 async fn create_or_update_roms(
     connection: &mut SqliteConnection,
+    progress_bar: &ProgressBar,
     roms_xml: &[RomXml],
+    mut bios: bool,
     game_id: i64,
 ) -> Vec<i64> {
     let mut orphan_romfile_ids: Vec<i64> = Vec::new();
     for rom_xml in roms_xml {
+        // skip nodump roms
+        if rom_xml.status.is_some() && rom_xml.status.as_ref().unwrap() == "nodump" {
+            continue;
+        }
+        // find parent rom if needed
+        let mut parent_id = None;
+        if rom_xml.merge.is_some() {
+            let game = find_game_by_id(connection, game_id).await;
+            // try cloneof first, or romof if there is no cloneof
+            let mut parent_rom = find_rom_by_size_and_crc_and_game_id(
+                connection,
+                rom_xml.size,
+                rom_xml.crc.as_ref().unwrap(),
+                game.parent_id.or(game.bios_id).unwrap(),
+            )
+            .await;
+            // try romof next
+            if parent_rom.is_none() && game.bios_id.is_some() {
+                parent_rom = find_rom_by_size_and_crc_and_game_id(
+                    connection,
+                    rom_xml.size,
+                    rom_xml.crc.as_ref().unwrap(),
+                    game.bios_id.unwrap(),
+                )
+                .await;
+            }
+            if parent_rom.is_some() {
+                let parent_rom = parent_rom.unwrap();
+                bios = parent_rom.bios;
+                parent_id = parent_rom.parent_id.or(Some(parent_rom.id));
+            } else {
+                progress_bar.println(&format!(
+                    "Rom \"{}\" not found in game \"{}\" parent/bios, please fix your DAT file",
+                    rom_xml.name, game.name
+                ));
+            }
+        }
         let rom = find_rom_by_name_and_game_id(connection, &rom_xml.name, game_id).await;
         match rom {
             Some(rom) => {
-                update_rom(connection, rom.id, &rom_xml, game_id).await;
-                if rom_xml.size != rom.size || rom_xml.crc != rom.crc {
+                update_rom(connection, rom.id, rom_xml, bios, game_id, parent_id).await;
+                if rom_xml.size != rom.size || rom_xml.crc.as_ref().unwrap() != &rom.crc {
                     if let Some(romfile_id) = rom.romfile_id {
                         orphan_romfile_ids.push(romfile_id);
                         update_rom_romfile(connection, rom.id, None).await;
@@ -359,7 +474,7 @@ async fn create_or_update_roms(
                 }
                 rom.id
             }
-            None => create_rom(connection, &rom_xml, game_id).await,
+            None => create_rom(connection, rom_xml, bios, game_id, parent_id).await,
         };
     }
     orphan_romfile_ids
@@ -379,7 +494,7 @@ async fn delete_old_games(
         .collect();
     for game in games {
         orphan_romfile_ids.extend(
-            find_roms_by_game_id(connection, game.id)
+            find_roms_by_game_id_no_parents(connection, game.id)
                 .await
                 .into_iter()
                 .filter_map(|rom| rom.romfile_id),
@@ -394,17 +509,16 @@ async fn delete_old_roms(
     roms_xml: &[RomXml],
     game_id: i64,
 ) -> Vec<i64> {
-    let rom_names_xml: Vec<&String> = roms_xml.iter().map(|rom_xml| &rom_xml.name).collect();
     let rom_names_romfile_ids: Vec<(String, Option<i64>)> =
-        find_roms_by_game_id(connection, game_id)
+        find_roms_by_game_id_no_parents(connection, game_id)
             .await
             .into_par_iter()
             .map(|rom| (rom.name, rom.romfile_id))
             .collect();
     let mut orphan_romfile_ids: Vec<i64> = Vec::new();
     for (rom_name, rom_romfile_id) in &rom_names_romfile_ids {
-        if !rom_names_xml.contains(&rom_name) {
-            delete_rom_by_name_and_game_id(connection, &rom_name, game_id).await;
+        if !roms_xml.iter().any(|rom_xml| &rom_xml.name == rom_name) {
+            delete_rom_by_name_and_game_id(connection, rom_name, game_id).await;
             if let Some(romfile_id) = rom_romfile_id {
                 orphan_romfile_ids.push(*romfile_id);
             }
@@ -456,7 +570,7 @@ mod test {
         let mut connection = pool.acquire().await.unwrap();
 
         let dat_path = test_directory.join("Test System (20200721).dat");
-        let (datfile_xml, detector_xml) = parse_dat(&progress_bar, &dat_path, false).unwrap();
+        let (datfile_xml, detector_xml) = parse_dat(&progress_bar, &dat_path, false).await.unwrap();
 
         // when
         import_dat(
@@ -465,6 +579,7 @@ mod test {
             &datfile_xml,
             &detector_xml,
             false,
+            false,
         )
         .await
         .unwrap();
@@ -491,7 +606,7 @@ mod test {
         let mut connection = pool.acquire().await.unwrap();
 
         let dat_path = test_directory.join("Test System (20200721) (Parent-Clone).dat");
-        let (datfile_xml, detector_xml) = parse_dat(&progress_bar, &dat_path, false).unwrap();
+        let (datfile_xml, detector_xml) = parse_dat(&progress_bar, &dat_path, false).await.unwrap();
 
         // when
         import_dat(
@@ -500,6 +615,7 @@ mod test {
             &datfile_xml,
             &detector_xml,
             false,
+            false,
         )
         .await
         .unwrap();
@@ -526,7 +642,7 @@ mod test {
         let mut connection = pool.acquire().await.unwrap();
 
         let dat_path = test_directory.join("Test System (20210402) (Headered).dat");
-        let (datfile_xml, detector_xml) = parse_dat(&progress_bar, &dat_path, false).unwrap();
+        let (datfile_xml, detector_xml) = parse_dat(&progress_bar, &dat_path, false).await.unwrap();
 
         // when
         import_dat(
@@ -535,6 +651,7 @@ mod test {
             &datfile_xml,
             &detector_xml,
             false,
+            false,
         )
         .await
         .unwrap();
@@ -544,7 +661,7 @@ mod test {
         assert_eq!(systems.len(), 1);
 
         let system = systems.get(0).unwrap();
-        assert_eq!(system.name, "Test System");
+        assert_eq!(system.name, "Test System (Headered)");
 
         assert!(find_header_by_system_id(&mut connection, system.id)
             .await
@@ -565,7 +682,7 @@ mod test {
         let mut connection = pool.acquire().await.unwrap();
 
         let dat_path = test_directory.join("Test System (20210402) (Headered).dat");
-        let (datfile_xml, detector_xml) = parse_dat(&progress_bar, &dat_path, true).unwrap();
+        let (datfile_xml, detector_xml) = parse_dat(&progress_bar, &dat_path, true).await.unwrap();
 
         // when
         import_dat(
@@ -574,6 +691,7 @@ mod test {
             &datfile_xml,
             &detector_xml,
             false,
+            false,
         )
         .await
         .unwrap();
@@ -583,7 +701,7 @@ mod test {
         assert_eq!(systems.len(), 1);
 
         let system = systems.get(0).unwrap();
-        assert_eq!(system.name, "Test System");
+        assert_eq!(system.name, "Test System (Headered)");
 
         assert!(find_header_by_system_id(&mut connection, system.id)
             .await
@@ -593,6 +711,46 @@ mod test {
         assert_eq!(find_roms(&mut connection).await.len(), 1);
     }
 
+    #[async_std::test]
+    async fn test_import_dat_headered_embedded() {
+        // given
+        let test_directory = Path::new("test");
+        let progress_bar = ProgressBar::hidden();
+
+        let db_file = NamedTempFile::new().unwrap();
+        let pool = establish_connection(db_file.path().to_str().unwrap()).await;
+        let mut connection = pool.acquire().await.unwrap();
+
+        let dat_path = test_directory.join("Test System (20211124) (Headered) (Embedded).dat");
+        let (datfile_xml, detector_xml) = parse_dat(&progress_bar, &dat_path, false).await.unwrap();
+
+        // when
+        import_dat(
+            &mut connection,
+            &progress_bar,
+            &datfile_xml,
+            &detector_xml,
+            false,
+            false,
+        )
+        .await
+        .unwrap();
+
+        // then
+        let systems = find_systems(&mut connection).await;
+        assert_eq!(systems.len(), 1);
+
+        let system = systems.get(0).unwrap();
+        assert_eq!(system.name, "Test System (Headered) (Embedded)");
+
+        assert!(find_header_by_system_id(&mut connection, system.id)
+            .await
+            .is_some());
+
+        assert_eq!(find_games(&mut connection).await.len(), 1);
+        assert_eq!(find_roms(&mut connection).await.len(), 1);
+    }
+
     #[async_std::test]
     async fn test_import_updated_dat() {
         // given
@@ -606,7 +764,7 @@ mod test {
         let mut connection = pool.acquire().await.unwrap();
 
         let dat_path = test_directory.join("Test System (20200721).dat");
-        let (datfile_xml, detector_xml) = parse_dat(&progress_bar, &dat_path, false).unwrap();
+        let (datfile_xml, detector_xml) = parse_dat(&progress_bar, &dat_path, false).await.unwrap();
 
         let rom_directory = TempDir::new_in(&test_directory).unwrap();
         set_rom_directory(PathBuf::from(rom_directory.path()));
@@ -619,6 +777,7 @@ mod test {
             &datfile_xml,
             &detector_xml,
             false,
+            false,
         )
         .await
         .unwrap();
@@ -650,7 +809,7 @@ mod test {
         }
 
         let dat_path = test_directory.join("Test System (20210401).dat");
-        let (datfile_xml, detector_xml) = parse_dat(&progress_bar, &dat_path, false).unwrap();
+        let (datfile_xml, detector_xml) = parse_dat(&progress_bar, &dat_path, false).await.unwrap();
 
         // when
         import_dat(
@@ -659,6 +818,7 @@ mod test {
             &datfile_xml,
             &detector_xml,
             false,
+            false,
         )
         .await
         .unwrap();
@@ -717,7 +877,7 @@ mod test {
         let mut connection = pool.acquire().await.unwrap();
 
         let dat_path = test_directory.join("Test System (20200721).dat");
-        let (datfile_xml, detector_xml) = parse_dat(&progress_bar, &dat_path, false).unwrap();
+        let (datfile_xml, detector_xml) = parse_dat(&progress_bar, &dat_path, false).await.unwrap();
 
         import_dat(
             &mut connection,
@@ -725,12 +885,13 @@ mod test {
             &datfile_xml,
             &detector_xml,
             false,
+            false,
         )
         .await
         .unwrap();
 
         let dat_path = test_directory.join("Test System (20000000).dat");
-        let (datfile_xml, detector_xml) = parse_dat(&progress_bar, &dat_path, false).unwrap();
+        let (datfile_xml, detector_xml) = parse_dat(&progress_bar, &dat_path, false).await.unwrap();
 
         // when
         import_dat(
@@ -739,6 +900,7 @@ mod test {
             &datfile_xml,
             &detector_xml,
             false,
+            false,
         )
         .await
         .unwrap();
@@ -765,7 +927,7 @@ mod test {
         let mut connection = pool.acquire().await.unwrap();
 
         let dat_path = test_directory.join("Test System (20200721).dat");
-        let (datfile_xml, detector_xml) = parse_dat(&progress_bar, &dat_path, false).unwrap();
+        let (datfile_xml, detector_xml) = parse_dat(&progress_bar, &dat_path, false).await.unwrap();
 
         import_dat(
             &mut connection,
@@ -773,12 +935,13 @@ mod test {
             &datfile_xml,
             &detector_xml,
             false,
+            false,
         )
         .await
         .unwrap();
 
         let dat_path = test_directory.join("Test System (20000000).dat");
-        let (datfile_xml, detector_xml) = parse_dat(&progress_bar, &dat_path, false).unwrap();
+        let (datfile_xml, detector_xml) = parse_dat(&progress_bar, &dat_path, false).await.unwrap();
 
         // when
         import_dat(
@@ -786,6 +949,7 @@ mod test {
             &progress_bar,
             &datfile_xml,
             &detector_xml,
+            false,
             true,
         )
         .await
