@@ -1,73 +1,72 @@
-use super::chdman::*;
 use super::config::*;
 use super::database::*;
-use super::maxcso::*;
 use super::model::*;
-use super::prompt::*;
-use super::sevenzip::*;
 use super::progress::*;
+use super::prompt::*;
 use super::util::*;
 use super::SimpleResult;
 use async_std::path::{Path, PathBuf};
-use clap::{App, Arg, ArgMatches, SubCommand};
+use clap::{App, Arg, ArgMatches};
 use indicatif::ProgressBar;
 use rayon::prelude::*;
 use shiratsu_naming::naming::nointro::{NoIntroName, NoIntroToken};
 use shiratsu_naming::naming::TokenizedName;
 use shiratsu_naming::region::Region;
 use sqlx::sqlite::SqliteConnection;
+use std::cmp::Ordering;
 use std::collections::HashMap;
+use std::ffi::OsString;
 
-pub fn subcommand<'a, 'b>() -> App<'a, 'b> {
-    SubCommand::with_name("sort-roms")
-        .about("Sorts ROM files according to region and version preferences")
+pub fn subcommand<'a>() -> App<'a> {
+    App::new("sort-roms")
+        .about("Sort ROM files according to region and version preferences")
         .arg(
-            Arg::with_name("REGIONS_ALL")
-                .short("r")
+            Arg::new("REGIONS_ALL")
+                .short('r')
                 .long("regions")
-                .help("Sets the regions to keep (unordered)")
+                .help("Set the regions to keep (unordered)")
                 .required(false)
                 .takes_value(true)
-                .multiple(true),
+                .multiple_values(true),
         )
         .arg(
-            Arg::with_name("REGIONS_ONE")
-                .short("g")
+            Arg::new("REGIONS_ONE")
+                .short('g')
                 .long("1g1r")
-                .help("Sets the 1G1R regions to keep (ordered)")
+                .help("Set the 1G1R regions to keep (ordered)")
                 .required(false)
                 .takes_value(true)
-                .multiple(true),
+                .multiple_values(true),
         )
         .arg(
-            Arg::with_name("MISSING")
-                .short("m")
+            Arg::new("MISSING")
+                .short('m')
                 .long("missing")
-                .help("Shows missing games")
+                .help("Show missing games")
                 .required(false),
         )
         .arg(
-            Arg::with_name("ALL")
-                .short("a")
+            Arg::new("ALL")
+                .short('a')
                 .long("all")
-                .help("Sorts all systems")
+                .help("Sort all systems")
                 .required(false),
         )
         .arg(
-            Arg::with_name("YES")
-                .short("y")
+            Arg::new("YES")
+                .short('y')
                 .long("yes")
-                .help("Automatically says yes to prompts")
+                .help("Automatically say yes to prompts")
                 .required(false),
         )
 }
 
 pub async fn main(
     connection: &mut SqliteConnection,
-    matches: &ArgMatches<'_>,
+    matches: &ArgMatches,
     progress_bar: &ProgressBar,
 ) -> SimpleResult<()> {
-    let systems = prompt_for_systems(connection, None, matches.is_present("ALL")).await?;
+    let systems = prompt_for_systems(connection, None, false, matches.is_present("ALL")).await?;
 
     progress_bar.enable_steady_tick(100);
 
@@ -80,7 +79,7 @@ pub async fn main(
         sort_system(
             connection,
             matches,
-            &progress_bar,
+            progress_bar,
             &system,
             &all_regions,
             &one_regions,
@@ -103,7 +102,7 @@ pub async fn main(
 
 pub async fn get_regions(
     connection: &mut SqliteConnection,
-    matches: &ArgMatches<'_>,
+    matches: &ArgMatches,
     key: &str,
 ) -> Vec<Region> {
     let all_regions: Vec<String> = if matches.is_present(key) {
@@ -122,7 +121,7 @@ pub async fn get_regions(
 
 async fn sort_system(
     connection: &mut SqliteConnection,
-    matches: &ArgMatches<'_>,
+    matches: &ArgMatches,
     progress_bar: &ProgressBar,
     system: &System,
     all_regions: &[Region],
@@ -136,6 +135,7 @@ async fn sort_system(
     let mut all_regions_games: Vec<Game> = Vec::new();
     let mut one_region_games: Vec<Game> = Vec::new();
     let mut ignored_games: Vec<Game> = Vec::new();
+    let mut missing_games: Vec<Game> = Vec::new();
     let mut romfile_moves: Vec<(&Romfile, String)> = Vec::new();
 
     let romfiles = find_romfiles_by_system_id(connection, system.id).await;
@@ -145,7 +145,7 @@ async fn sort_system(
         .collect();
 
     // 1G1R mode
-    if !one_regions.is_empty() {
+    if !system.arcade && !one_regions.is_empty() {
         let parent_games = find_parent_games_by_system_id(connection, system.id).await;
         let clone_games = find_clone_games_by_system_id(connection, system.id).await;
 
@@ -160,6 +160,8 @@ async fn sort_system(
         for parent_game in parent_games {
             if clone_games_by_parent_id.contains_key(&parent_game.id) {
                 games = clone_games_by_parent_id.remove(&parent_game.id).unwrap();
+                // put newer releases first
+                games.sort_by(sort_games_by_version_or_name_desc);
             } else {
                 games = Vec::new();
             }
@@ -167,18 +169,19 @@ async fn sort_system(
 
             // trim ignored games
             if !ignored_releases.is_empty() || !ignored_flags.is_empty() {
-                let (mut unwanted_games, regular_games) =
-                    trim_games(games, ignored_releases, ignored_flags);
-                ignored_games.append(&mut unwanted_games);
-                games = regular_games;
+                let (mut left_games, right_games) =
+                    trim_ignored_games(games, ignored_releases, ignored_flags, system.arcade);
+                ignored_games.append(&mut left_games);
+                games = right_games;
             }
 
             // find the one game we want to keep, if any
             for region in one_regions {
                 let i = games.iter().position(|game| {
-                    Region::try_from_tosec_region(&game.regions)
-                        .unwrap()
-                        .contains(region)
+                    game.complete
+                        && Region::try_from_tosec_region(&game.regions)
+                            .unwrap_or_default()
+                            .contains(region)
                 });
                 if i.is_some() {
                     one_region_games.push(games.remove(i.unwrap()));
@@ -191,36 +194,44 @@ async fn sort_system(
                 let game = games.remove(0);
                 let region_in_all_regions = all_regions.iter().any(|region| {
                     Region::try_from_tosec_region(&game.regions)
-                        .unwrap()
+                        .unwrap_or_default()
                         .contains(region)
                 });
                 if region_in_all_regions {
-                    all_regions_games.push(game);
+                    if game.complete {
+                        all_regions_games.push(game);
+                    } else {
+                        missing_games.push(game);
+                    }
                 } else {
                     ignored_games.push(game);
                 }
             }
         }
     // Regions mode
-    } else if !all_regions.is_empty() {
+    } else if !system.arcade && !all_regions.is_empty() {
         games = find_games_by_system_id(connection, system.id).await;
 
         // trim ignored games
         if !ignored_releases.is_empty() || !ignored_flags.is_empty() {
-            let (mut unwanted_games, regular_games) =
-                trim_games(games, ignored_releases, ignored_flags);
-            ignored_games.append(&mut unwanted_games);
-            games = regular_games;
+            let (mut left_games, right_games) =
+                trim_ignored_games(games, ignored_releases, ignored_flags, system.arcade);
+            ignored_games.append(&mut left_games);
+            games = right_games;
         }
 
         for game in games {
             let region_in_all_regions = all_regions.iter().any(|region| {
                 Region::try_from_tosec_region(&game.regions)
-                    .unwrap()
+                    .unwrap_or_default()
                     .contains(region)
             });
             if region_in_all_regions {
-                all_regions_games.push(game);
+                if game.complete {
+                    all_regions_games.push(game);
+                } else {
+                    missing_games.push(game);
+                }
             } else {
                 ignored_games.push(game);
             }
@@ -230,26 +241,40 @@ async fn sort_system(
 
         // trim ignored games
         if !ignored_releases.is_empty() || !ignored_flags.is_empty() {
-            let (mut unwanted_games, regular_games) =
-                trim_games(games, ignored_releases, ignored_flags);
-            ignored_games.append(&mut unwanted_games);
-            games = regular_games;
+            let (mut left_games, right_games) =
+                trim_ignored_games(games, ignored_releases, ignored_flags, system.arcade);
+            ignored_games.append(&mut left_games);
+            games = right_games;
         }
 
-        all_regions_games.append(&mut games);
+        for game in games {
+            if game.complete {
+                all_regions_games.push(game);
+            } else {
+                missing_games.push(game)
+            }
+        }
     }
 
     if matches.is_present("MISSING") {
-        let mut game_ids: Vec<i64> = Vec::new();
-        game_ids.append(&mut all_regions_games.iter().map(|game| game.id).collect());
-        game_ids.append(&mut one_region_games.iter().map(|game| game.id).collect());
-        let missing_roms: Vec<Rom> =
-            find_roms_without_romfile_by_game_ids(connection, &game_ids).await;
+        let mut missing_roms: Vec<Rom> = find_roms_without_romfile_by_game_ids(
+            connection,
+            &missing_games
+                .par_iter()
+                .map(|game| game.id)
+                .collect::<Vec<i64>>(),
+        )
+        .await;
 
         if !missing_roms.is_empty() {
             progress_bar.println("Missing:");
+            missing_roms.sort_by_key(|rom| rom.game_id);
             for rom in missing_roms {
-                progress_bar.println(&format!("{} [{}]", rom.name, rom.crc));
+                let game = missing_games
+                    .iter()
+                    .find(|&game| game.id == rom.game_id)
+                    .unwrap();
+                progress_bar.println(&format!("{} ({}) [{}]", rom.name, game.name, rom.crc));
             }
         } else {
             progress_bar.println("No missing ROMs");
@@ -264,7 +289,7 @@ async fn sort_system(
 
     let mut changes = 0;
 
-    // process all_region_games
+    // process all region games
     changes += update_games_sorting(
         &mut transaction,
         &all_regions_games
@@ -277,14 +302,16 @@ async fn sort_system(
     romfile_moves.append(
         &mut sort_games(
             &mut transaction,
+            progress_bar,
+            system,
             all_regions_games,
             &system_directory,
             &romfiles_by_id,
         )
-        .await,
+        .await?,
     );
 
-    // process one_region_games
+    // process one region games
     changes += update_games_sorting(
         &mut transaction,
         &one_region_games
@@ -297,14 +324,38 @@ async fn sort_system(
     romfile_moves.append(
         &mut sort_games(
             &mut transaction,
+            progress_bar,
+            system,
             one_region_games,
             &one_region_directory,
             &romfiles_by_id,
         )
-        .await,
+        .await?,
     );
 
-    // process trash_games
+    // process missing games
+    changes += update_games_sorting(
+        &mut transaction,
+        &missing_games
+            .iter()
+            .map(|game| game.id)
+            .collect::<Vec<i64>>(),
+        Sorting::AllRegions,
+    )
+    .await;
+    romfile_moves.append(
+        &mut sort_games(
+            &mut transaction,
+            progress_bar,
+            system,
+            missing_games,
+            &system_directory,
+            &romfiles_by_id,
+        )
+        .await?,
+    );
+
+    // process ignored games
     changes += update_games_sorting(
         &mut transaction,
         &ignored_games
@@ -317,11 +368,13 @@ async fn sort_system(
     romfile_moves.append(
         &mut sort_games(
             &mut transaction,
+            progress_bar,
+            system,
             ignored_games,
             &trash_directory,
             &romfiles_by_id,
         )
-        .await,
+        .await?,
     );
 
     if !romfile_moves.is_empty() {
@@ -363,7 +416,6 @@ async fn sort_system(
         progress_bar.set_style(get_none_progress_style());
         progress_bar.enable_steady_tick(100);
         progress_bar.set_message("Computing system completion");
-        update_games_by_system_id_mark_complete(connection, system.id).await;
         update_games_by_system_id_mark_incomplete(connection, system.id).await;
         update_system_mark_complete(connection, system.id).await;
         update_system_mark_incomplete(connection, system.id).await;
@@ -374,15 +426,17 @@ async fn sort_system(
 
 async fn sort_games<'a, P: AsRef<Path>>(
     connection: &mut SqliteConnection,
+    progress_bar: &ProgressBar,
+    system: &System,
     games: Vec<Game>,
     directory: &P,
     romfiles_by_id: &'a HashMap<i64, Romfile>,
-) -> Vec<(&'a Romfile, String)> {
+) -> SimpleResult<Vec<(&'a Romfile, String)>> {
     let mut romfile_moves: Vec<(&Romfile, String)> = Vec::new();
 
     let roms = find_roms_with_romfile_by_game_ids(
         connection,
-        &games
+        games
             .iter()
             .map(|game| game.id)
             .collect::<Vec<i64>>()
@@ -403,68 +457,131 @@ async fn sort_games<'a, P: AsRef<Path>>(
             None => continue,
         };
         let rom_count = roms.len();
-        romfile_moves.append(
-            &mut roms
-                .iter()
-                .map(|rom| {
-                    let romfile = romfiles_by_id.get(&rom.romfile_id.unwrap()).unwrap();
-                    let new_path = String::from(
-                        compute_new_path(&game, &rom, &romfile, rom_count, directory)
-                            .as_os_str()
-                            .to_str()
-                            .unwrap(),
-                    );
-                    (romfile, new_path)
-                })
-                .filter(|(romfile, new_path)| &romfile.path != new_path)
-                .collect(),
-        );
+        for rom in roms {
+            let romfile = romfiles_by_id.get(&rom.romfile_id.unwrap()).unwrap();
+            let new_path = String::from(
+                compute_new_path(
+                    progress_bar,
+                    system,
+                    &game,
+                    rom,
+                    romfile,
+                    rom_count,
+                    directory,
+                )
+                .await?
+                .as_os_str()
+                .to_str()
+                .unwrap(),
+            );
+            if romfile.path != new_path {
+                romfile_moves.push((romfile, new_path))
+            }
+        }
     }
 
-    romfile_moves
+    Ok(romfile_moves)
 }
 
-fn trim_games(
+fn trim_ignored_games(
     games: Vec<Game>,
     ignored_releases: &[&str],
     ignored_flags: &[&str],
+    arcade: bool,
 ) -> (Vec<Game>, Vec<Game>) {
     // TODO: use drain_filter when it hits stable
-    games.into_par_iter().partition(|game| {
-        if let Ok(name) = NoIntroName::try_parse(&game.name) {
-            for token in name.iter() {
-                if let NoIntroToken::Release(release, _) = token {
-                    if ignored_releases.contains(release) {
-                        return true;
-                    }
+    if arcade {
+        games.into_iter().partition(|game| {
+            let comment = match game.comment.as_ref() {
+                Some(comment) => comment,
+                None => "",
+            };
+            for ignored_release in ignored_releases {
+                if comment.contains(ignored_release) {
+                    return true;
                 }
-                if let NoIntroToken::Flag(_, flags) = token {
-                    for flag in flags.split(", ") {
-                        if ignored_flags.contains(&flag) {
+            }
+            false
+        })
+    } else {
+        games.into_iter().partition(|game| {
+            if let Ok(name) = NoIntroName::try_parse(&game.name) {
+                for token in name.iter() {
+                    if let NoIntroToken::Release(release, _) = token {
+                        if ignored_releases.contains(release) {
                             return true;
                         }
                     }
+                    if let NoIntroToken::Flag(_, flags) = token {
+                        for flag in flags.split(", ") {
+                            if ignored_flags.contains(&flag) {
+                                return true;
+                            }
+                        }
+                    }
                 }
             }
-        }
-        false
-    })
+            false
+        })
+    }
 }
 
-fn compute_new_path<P: AsRef<Path>>(
+fn sort_games_by_version_or_name_desc(game_a: &Game, game_b: &Game) -> Ordering {
+    let mut version_a = None;
+    let mut version_b = None;
+    if let Ok(name) = NoIntroName::try_parse(&game_a.name) {
+        for token in name.iter() {
+            if let NoIntroToken::Version(version) = token {
+                version_a = Some(version.to_owned());
+            }
+        }
+    }
+    if let Ok(name) = NoIntroName::try_parse(&game_b.name) {
+        for token in name.iter() {
+            if let NoIntroToken::Version(version) = token {
+                version_b = Some(version.to_owned());
+            }
+        }
+    }
+    if let (Some(version_a), Some(version_b)) = (version_a.as_ref(), version_b.as_ref()) {
+        return version_b.partial_cmp(version_a).unwrap();
+    }
+    if version_a.is_some() {
+        return Ordering::Less;
+    }
+    if version_b.is_some() {
+        return Ordering::Greater;
+    }
+    game_b.name.partial_cmp(&game_a.name).unwrap()
+}
+
+async fn compute_new_path<P: AsRef<Path>>(
+    progress_bar: &ProgressBar,
+    system: &System,
     game: &Game,
     rom: &Rom,
     romfile: &Romfile,
     rom_count: usize,
     directory: &P,
-) -> PathBuf {
+) -> SimpleResult<PathBuf> {
     let romfile_path = Path::new(&romfile.path);
-    let romfile_extension = romfile_path.extension().unwrap().to_str().unwrap();
+    let romfile_extension = romfile_path
+        .extension()
+        .unwrap_or(&OsString::new())
+        .to_str()
+        .unwrap()
+        .to_lowercase();
     let mut new_romfile_path: PathBuf;
 
-    if ARCHIVE_EXTENSIONS.contains(&romfile_extension) {
+    if ARCHIVE_EXTENSIONS.contains(&romfile_extension.as_str()) {
         new_romfile_path = directory.as_ref().join(match rom_count {
-            1 => format!("{}.{}", &rom.name, &romfile_extension),
+            1 => {
+                if system.arcade {
+                    format!("{}.{}", &game.name, &romfile_extension)
+                } else {
+                    format!("{}.{}", &rom.name, &romfile_extension)
+                }
+            }
             _ => format!("{}.{}", &game.name, &romfile_extension),
         });
     } else if romfile_extension == CHD_EXTENSION {
@@ -479,10 +596,19 @@ fn compute_new_path<P: AsRef<Path>>(
     } else if romfile_extension == CSO_EXTENSION {
         new_romfile_path = directory.as_ref().join(&rom.name);
         new_romfile_path.set_extension(&romfile_extension);
-    } else {
+    } else if romfile_extension == RVZ_EXTENSION {
         new_romfile_path = directory.as_ref().join(&rom.name);
+        new_romfile_path.set_extension(&romfile_extension);
+    } else {
+        new_romfile_path = match system.arcade {
+            true => {
+                create_directory(progress_bar, &directory.as_ref().join(&game.name), true).await?;
+                directory.as_ref().join(&game.name).join(&rom.name)
+            }
+            false => directory.as_ref().join(&rom.name),
+        };
     }
-    new_romfile_path
+    Ok(new_romfile_path)
 }
 
 #[cfg(test)]
@@ -576,48 +702,60 @@ mod test {
     }
 
     #[async_std::test]
-    async fn test_trim_games_should_discard_ignored_games() {
+    async fn test_trim_ignored_games_should_discard_ignored_games() {
         // given
         let games = vec![
             Game {
                 id: 1,
                 name: String::from("Game (USA)"),
                 description: String::from(""),
+                comment: None,
+                bios: false,
                 regions: String::from(""),
-                sorting: Sorting::AllRegions,
-                complete: false,
+                sorting: Sorting::AllRegions as i64,
+                complete: true,
                 system_id: 1,
                 parent_id: None,
+                bios_id: None,
             },
             Game {
                 id: 2,
                 name: String::from("Game (USA) (Beta)"),
                 description: String::from(""),
+                comment: None,
+                bios: false,
                 regions: String::from(""),
-                sorting: Sorting::AllRegions,
-                complete: false,
+                sorting: Sorting::AllRegions as i64,
+                complete: true,
                 system_id: 1,
                 parent_id: None,
+                bios_id: None,
             },
             Game {
                 id: 3,
                 name: String::from("Game (USA) (Beta 1)"),
                 description: String::from(""),
+                comment: None,
+                bios: false,
                 regions: String::from(""),
-                sorting: Sorting::AllRegions,
-                complete: false,
+                sorting: Sorting::AllRegions as i64,
+                complete: true,
                 system_id: 1,
                 parent_id: None,
+                bios_id: None,
             },
             Game {
                 id: 4,
                 name: String::from("Game (USA) (Virtual Console, Switch Online)"),
                 description: String::from(""),
+                comment: None,
+                bios: false,
                 regions: String::from(""),
-                sorting: Sorting::AllRegions,
-                complete: false,
+                sorting: Sorting::AllRegions as i64,
+                complete: true,
                 system_id: 1,
                 parent_id: None,
+                bios_id: None,
             },
         ];
 
@@ -625,7 +763,8 @@ mod test {
         let ignored_flags = vec!["Virtual Console"];
 
         // when
-        let (ignored_games, regular_games) = trim_games(games, &ignored_releases, &ignored_flags);
+        let (ignored_games, regular_games) =
+            trim_ignored_games(games, &ignored_releases, &ignored_flags, false);
 
         // then
         assert_eq!(ignored_games.len(), 3);
@@ -633,30 +772,157 @@ mod test {
         assert_eq!(regular_games.get(0).unwrap().name, "Game (USA)")
     }
 
+    #[async_std::test]
+    async fn test_sort_games_by_version_or_name_between_two_revisions() {
+        // given
+        let game_a = Game {
+            id: 1,
+            name: String::from("Game (USA) (Rev 1)"),
+            description: String::from(""),
+            comment: None,
+            bios: false,
+            regions: String::from(""),
+            sorting: Sorting::AllRegions as i64,
+            complete: true,
+            system_id: 1,
+            parent_id: None,
+            bios_id: None,
+        };
+        let game_b = Game {
+            id: 1,
+            name: String::from("Game (USA) (Rev 2)"),
+            description: String::from(""),
+            comment: None,
+            bios: false,
+            regions: String::from(""),
+            sorting: Sorting::AllRegions as i64,
+            complete: true,
+            system_id: 1,
+            parent_id: None,
+            bios_id: None,
+        };
+
+        // when
+        let ordering = sort_games_by_version_or_name_desc(&game_a, &game_b);
+
+        // then
+        assert_eq!(ordering, Ordering::Greater);
+    }
+
+    #[async_std::test]
+    async fn test_sort_games_by_version_or_name_between_vanilla_and_revision() {
+        // given
+        let game_a = Game {
+            id: 1,
+            name: String::from("Game (USA)"),
+            description: String::from(""),
+            comment: None,
+            bios: false,
+            regions: String::from(""),
+            sorting: Sorting::AllRegions as i64,
+            complete: true,
+            system_id: 1,
+            parent_id: None,
+            bios_id: None,
+        };
+        let game_b = Game {
+            id: 1,
+            name: String::from("Game (USA) (Rev 2)"),
+            description: String::from(""),
+            comment: None,
+            bios: false,
+            regions: String::from(""),
+            sorting: Sorting::AllRegions as i64,
+            complete: true,
+            system_id: 1,
+            parent_id: None,
+            bios_id: None,
+        };
+
+        // when
+        let ordering = sort_games_by_version_or_name_desc(&game_a, &game_b);
+
+        // then
+        assert_eq!(ordering, Ordering::Greater);
+    }
+
+    #[async_std::test]
+    async fn test_sort_games_by_version_or_name_between_revision_and_vanilla() {
+        // given
+        let game_a = Game {
+            id: 1,
+            name: String::from("Game (USA) (Rev 2"),
+            description: String::from(""),
+            comment: None,
+            bios: false,
+            regions: String::from(""),
+            sorting: Sorting::AllRegions as i64,
+            complete: true,
+            system_id: 1,
+            parent_id: None,
+            bios_id: None,
+        };
+        let game_b = Game {
+            id: 1,
+            name: String::from("Game (USA)"),
+            description: String::from(""),
+            comment: None,
+            bios: false,
+            regions: String::from(""),
+            sorting: Sorting::AllRegions as i64,
+            complete: true,
+            system_id: 1,
+            parent_id: None,
+            bios_id: None,
+        };
+
+        // when
+        let ordering = sort_games_by_version_or_name_desc(&game_a, &game_b);
+
+        // then
+        assert_eq!(ordering, Ordering::Less);
+    }
+
     #[async_std::test]
     async fn test_compute_new_path_archive_single_file() {
         // given
+        let progress_bar = ProgressBar::hidden();
         let test_directory = Path::new("test");
+        let system = System {
+            id: 1,
+            name: String::from("Test System"),
+            description: String::from(""),
+            version: String::from(""),
+            url: Some(String::from("")),
+            arcade: false,
+            merging: Merging::Split as i64,
+            complete: false,
+        };
         let game = Game {
             id: 1,
             name: String::from("game name"),
             description: String::from(""),
+            comment: None,
+            bios: false,
             regions: String::from(""),
-            sorting: Sorting::AllRegions,
+            sorting: Sorting::AllRegions as i64,
             complete: false,
             system_id: 1,
             parent_id: None,
+            bios_id: None,
         };
         let rom = Rom {
             id: 1,
             name: String::from("rom name.rom"),
+            bios: false,
             size: 1,
             crc: String::from(""),
-            md5: String::from(""),
-            sha1: String::from(""),
+            md5: Some(String::from("")),
+            sha1: Some(String::from("")),
             rom_status: None,
             game_id: 1,
             romfile_id: Some(1),
+            parent_id: None,
         };
         let romfile = Romfile {
             id: 1,
@@ -665,7 +931,17 @@ mod test {
         };
 
         // when
-        let path = compute_new_path(&game, &rom, &romfile, 1, &test_directory);
+        let path = compute_new_path(
+            &progress_bar,
+            &system,
+            &game,
+            &rom,
+            &romfile,
+            1,
+            &test_directory,
+        )
+        .await
+        .unwrap();
 
         // then
         assert_eq!(path, test_directory.join("rom name.rom.7z"));
@@ -674,27 +950,43 @@ mod test {
     #[async_std::test]
     async fn test_compute_new_path_archive_multiple_files() {
         // given
+        let progress_bar = ProgressBar::hidden();
         let test_directory = Path::new("test");
+        let system = System {
+            id: 1,
+            name: String::from("Test System"),
+            description: String::from(""),
+            version: String::from(""),
+            url: Some(String::from("")),
+            arcade: false,
+            merging: Merging::Split as i64,
+            complete: false,
+        };
         let game = Game {
             id: 1,
             name: String::from("game name"),
             description: String::from(""),
+            comment: None,
+            bios: false,
             regions: String::from(""),
-            sorting: Sorting::AllRegions,
+            sorting: Sorting::AllRegions as i64,
             complete: false,
             system_id: 1,
             parent_id: None,
+            bios_id: None,
         };
         let rom = Rom {
             id: 1,
             name: String::from("rom name.rom"),
+            bios: false,
             size: 1,
             crc: String::from(""),
-            md5: String::from(""),
-            sha1: String::from(""),
+            md5: Some(String::from("")),
+            sha1: Some(String::from("")),
             rom_status: None,
             game_id: 1,
             romfile_id: Some(1),
+            parent_id: None,
         };
         let romfile = Romfile {
             id: 1,
@@ -703,7 +995,17 @@ mod test {
         };
 
         // when
-        let path = compute_new_path(&game, &rom, &romfile, 2, &test_directory);
+        let path = compute_new_path(
+            &progress_bar,
+            &system,
+            &game,
+            &rom,
+            &romfile,
+            2,
+            &test_directory,
+        )
+        .await
+        .unwrap();
 
         // then
         assert_eq!(path, test_directory.join("game name.7z"));
@@ -712,27 +1014,43 @@ mod test {
     #[async_std::test]
     async fn test_compute_new_path_chd_single_file() {
         // given
+        let progress_bar = ProgressBar::hidden();
         let test_directory = Path::new("test");
+        let system = System {
+            id: 1,
+            name: String::from("Test System"),
+            description: String::from(""),
+            version: String::from(""),
+            url: Some(String::from("")),
+            arcade: false,
+            merging: Merging::Split as i64,
+            complete: false,
+        };
         let game = Game {
             id: 1,
             name: String::from("game name"),
             description: String::from(""),
+            comment: None,
+            bios: false,
             regions: String::from(""),
-            sorting: Sorting::AllRegions,
+            sorting: Sorting::AllRegions as i64,
             complete: false,
             system_id: 1,
             parent_id: None,
+            bios_id: None,
         };
         let rom = Rom {
             id: 1,
             name: String::from("rom name.bin"),
+            bios: false,
             size: 1,
             crc: String::from(""),
-            md5: String::from(""),
-            sha1: String::from(""),
+            md5: Some(String::from("")),
+            sha1: Some(String::from("")),
             rom_status: None,
             game_id: 1,
             romfile_id: Some(1),
+            parent_id: None,
         };
         let romfile = Romfile {
             id: 1,
@@ -741,7 +1059,17 @@ mod test {
         };
 
         // when
-        let path = compute_new_path(&game, &rom, &romfile, 2, &test_directory);
+        let path = compute_new_path(
+            &progress_bar,
+            &system,
+            &game,
+            &rom,
+            &romfile,
+            2,
+            &test_directory,
+        )
+        .await
+        .unwrap();
 
         // then
         assert_eq!(path, test_directory.join("rom name.chd"));
@@ -750,27 +1078,43 @@ mod test {
     #[async_std::test]
     async fn test_compute_new_path_chd_multiple_files() {
         // given
+        let progress_bar = ProgressBar::hidden();
         let test_directory = Path::new("test");
+        let system = System {
+            id: 1,
+            name: String::from("Test System"),
+            description: String::from(""),
+            version: String::from(""),
+            url: Some(String::from("")),
+            arcade: false,
+            merging: Merging::Split as i64,
+            complete: false,
+        };
         let game = Game {
             id: 1,
             name: String::from("game name"),
             description: String::from(""),
+            comment: None,
+            bios: false,
             regions: String::from(""),
-            sorting: Sorting::AllRegions,
+            sorting: Sorting::AllRegions as i64,
             complete: false,
             system_id: 1,
             parent_id: None,
+            bios_id: None,
         };
         let rom = Rom {
             id: 1,
             name: String::from("rom name.bin"),
+            bios: false,
             size: 1,
             crc: String::from(""),
-            md5: String::from(""),
-            sha1: String::from(""),
+            md5: Some(String::from("")),
+            sha1: Some(String::from("")),
             rom_status: None,
             game_id: 1,
             romfile_id: Some(1),
+            parent_id: None,
         };
         let romfile = Romfile {
             id: 1,
@@ -779,7 +1123,17 @@ mod test {
         };
 
         // when
-        let path = compute_new_path(&game, &rom, &romfile, 3, &test_directory);
+        let path = compute_new_path(
+            &progress_bar,
+            &system,
+            &game,
+            &rom,
+            &romfile,
+            3,
+            &test_directory,
+        )
+        .await
+        .unwrap();
 
         // then
         assert_eq!(path, test_directory.join("game name.chd"));
@@ -788,27 +1142,43 @@ mod test {
     #[async_std::test]
     async fn test_compute_new_path_cso() {
         // given
+        let progress_bar = ProgressBar::hidden();
         let test_directory = Path::new("test");
+        let system = System {
+            id: 1,
+            name: String::from("Test System"),
+            description: String::from(""),
+            version: String::from(""),
+            url: Some(String::from("")),
+            arcade: false,
+            merging: Merging::Split as i64,
+            complete: false,
+        };
         let game = Game {
             id: 1,
             name: String::from("game name"),
             description: String::from(""),
+            comment: None,
+            bios: false,
             regions: String::from(""),
-            sorting: Sorting::AllRegions,
+            sorting: Sorting::AllRegions as i64,
             complete: false,
             system_id: 1,
             parent_id: None,
+            bios_id: None,
         };
         let rom = Rom {
             id: 1,
             name: String::from("rom name.iso"),
+            bios: false,
             size: 1,
             crc: String::from(""),
-            md5: String::from(""),
-            sha1: String::from(""),
+            md5: Some(String::from("")),
+            sha1: Some(String::from("")),
             rom_status: None,
             game_id: 1,
             romfile_id: Some(1),
+            parent_id: None,
         };
         let romfile = Romfile {
             id: 1,
@@ -817,7 +1187,17 @@ mod test {
         };
 
         // when
-        let path = compute_new_path(&game, &rom, &romfile, 1, &test_directory);
+        let path = compute_new_path(
+            &progress_bar,
+            &system,
+            &game,
+            &rom,
+            &romfile,
+            1,
+            &test_directory,
+        )
+        .await
+        .unwrap();
 
         // then
         assert_eq!(path, test_directory.join("rom name.cso"));
@@ -826,27 +1206,43 @@ mod test {
     #[async_std::test]
     async fn test_compute_new_path_other() {
         // given
+        let progress_bar = ProgressBar::hidden();
         let test_directory = Path::new("test");
+        let system = System {
+            id: 1,
+            name: String::from("Test System"),
+            description: String::from(""),
+            version: String::from(""),
+            url: Some(String::from("")),
+            arcade: false,
+            merging: Merging::Split as i64,
+            complete: false,
+        };
         let game = Game {
             id: 1,
             name: String::from("game name"),
             description: String::from(""),
+            comment: None,
+            bios: false,
             regions: String::from(""),
-            sorting: Sorting::AllRegions,
+            sorting: Sorting::AllRegions as i64,
             complete: false,
             system_id: 1,
             parent_id: None,
+            bios_id: None,
         };
         let rom = Rom {
             id: 1,
             name: String::from("rom name.rom"),
+            bios: false,
             size: 1,
             crc: String::from(""),
-            md5: String::from(""),
-            sha1: String::from(""),
+            md5: Some(String::from("")),
+            sha1: Some(String::from("")),
             rom_status: None,
             game_id: 1,
             romfile_id: Some(1),
+            parent_id: None,
         };
         let romfile = Romfile {
             id: 1,
@@ -855,7 +1251,17 @@ mod test {
         };
 
         // when
-        let path = compute_new_path(&game, &rom, &romfile, 1, &test_directory);
+        let path = compute_new_path(
+            &progress_bar,
+            &system,
+            &game,
+            &rom,
+            &romfile,
+            1,
+            &test_directory,
+        )
+        .await
+        .unwrap();
 
         // then
         assert_eq!(path, test_directory.join("rom name.rom"));
